/******************************************************************************

Copyright 2023, Beijing ESWIN Computing Technology Co., Ltd.. All rights reserved.

******************************************************************************
File Name     : sample_aes_sm4.c
Version       : Initial Draft
Author        : Eswin
Created       : 2023
Last Modified :
Description   : sample for aes and sm4
Function List :
History       :
******************************************************************************/
#include "sample_cipher_common.h"

static ES_U8 aes_key[16] = {0x2B,0x7E,0x15,0x16,0x28,0xAE,0xD2,0xA6,0xAB,0xF7,0x15,0x88,0x09,0xCF,0x4F,0x3C};
static ES_U8 aes_IV[16]  = {0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07,0x08,0x09,0x0A,0x0B,0x0C,0x0D,0x0E,0x0F};

char  aes_src[80]={   0x01,0x11,0x22,0xCE,0x00,0x10,0x23,0x26,0x04,0xFE,0x85,0x47,0x23,0x24,0xF4,0xFE,\
                                0xB7,0x57,0x57,0x42,0x93,0x87,0x57,0x24,0x23,0x22,0xF4,0xFE,0x83,0x27,0x84,0xFE,\
                                0x92,0x07,0x23,0x24,0xF4,0xFE,0x05,0x67,0x83,0x27,0x84,0xFE,0xB3,0x47,0xF7,0x02,\
                                0x23,0x24,0xF4,0xFE,0x03,0x27,0x84,0xFE,0x83,0x27,0x44,0xFE,0xBA,0x97,0x23,0x26,\
                                0xF4,0xFE,0x83,0x27,0xC4,0xFE,0x3E,0x85,0x72,0x44,0x05,0x61,0x82,0x80,0x0,0x0};

static int do_symc_crypto(ES_CIPHER_CTRL_S *pstCipherCtrl, ES_U8 *pSrcVaddr, ES_U8 *pDestVaddr, int data_size, ES_CIPHER_CRYPTO_TYPE_E crypto)
{
	int ret = 0;
	ES_HANDLE hCipher;
	ES_CIPHER_CTRL_S stCipherCtrlGet;

	ES_CIPHER_Init();

	ret = ES_CIPHER_CreateHandle(&hCipher, pstCipherCtrl);
	if (ret != ES_SUCCESS) {
		printf("%s, create handle failed, err(%d)\n", __func__, ret);
		goto CIPHER_DEINIT;
	}

	ret = ES_CIPHER_GetHandleConfig(hCipher, &stCipherCtrlGet);
	if (ret != ES_SUCCESS) {
		printf("%s, get handle config failed, err(%d)\n", __func__, ret);
		goto DESTROY_HANDLE;
	}

	/* test if the configuration got via ES_CIPHER_GetHandleConfig is equal to the created configuration */
	ret = memcmp(pstCipherCtrl, &stCipherCtrlGet, sizeof(ES_CIPHER_CTRL_S));
	if (ret != 0) {
		printf("%s, get config is not equal to create config, err(%d)\n", __func__, ret);
		printf("---createhandle input:\n");
		print_data((ES_U8 *)pstCipherCtrl, sizeof(*pstCipherCtrl));

		printf("---GetHandleConfig output:\n");
		print_data((ES_U8 *)&stCipherCtrlGet, sizeof(stCipherCtrlGet));
		goto DESTROY_HANDLE;
	}

	/* Encrypt */
	if (crypto == OP_ENCRYPT) {
		ret = ES_CIPHER_Encrypt(hCipher, pSrcVaddr, pDestVaddr, data_size);
	}
	else if (crypto == OP_DECRYPT) {
		ret = ES_CIPHER_Decrypt(hCipher, pSrcVaddr, pDestVaddr, data_size);
	}
	else {
		printf("Invalid crypto(%d) type!!!\n", crypto);
		ret = -1;
	}
DESTROY_HANDLE:
	ES_CIPHER_DestroyHandle(hCipher);
CIPHER_DEINIT:
	ES_CIPHER_DeInit();
	return ret;
}

/* encrypt and decrypt with external user key */
static ES_S32 external_CBC_AES128(ES_VOID)
{
	ES_S32 ret = 0, err = 0;
	ES_CIPHER_CTRL_S stCipherCtrl;
	ES_U64 srcMemFd, destMemFd;
	ES_U64 decryptDestMemFd;
	ES_U8 *pSrcVaddr, *pDestVaddr;
	ES_U8 *pDecryptSrcVaddr, *pDecrytDestVaddr;
	int data_size = 0;

	printf("%s begin---\n", __func__);
	/* Init ES_CIPHER_CTRL_S */
	memset(&stCipherCtrl, 0, sizeof(ES_CIPHER_CTRL_S));

	stCipherCtrl.enKeySrc = ES_CIPHER_SYMC_KEY_SRC_USER;
	stCipherCtrl.enAlgMode = ES_CIPHER_ALG_MODE_AES_CBC;
	stCipherCtrl.enKeyLen = ES_CIPHER_KEY_AES_128BIT;
	stCipherCtrl.u32IvLen = 16;

	/* Init key and IV */
	memcpy(stCipherCtrl.u32Key, aes_key, ES_CIPHER_KEY_AES_128BIT);
	memcpy(stCipherCtrl.u32IV, aes_IV, stCipherCtrl.u32IvLen);

	data_size = sizeof(aes_src);
	/* alloc src and fill with test data, alloc dest buffer */
	ret = alloc_buf_and_fill_data(aes_src, data_size, &pSrcVaddr, &srcMemFd, HEAP_ALLOC);
	if (ret != 0) {
		ES_ERR_CIPHER("%s, failed to alloc mem for src data\n", __func__);
		return ES_FAILURE;
	}

	ret = alloc_buf_and_fill_data(NULL, data_size, &pDestVaddr, &destMemFd, USER_MALLOC);
	if (ret != 0) {
		ES_ERR_CIPHER("%s, failed to alloc mem for dst data\n", __func__);
		err++;
		goto FREE_SRC_BUF;
	}

	/* Encrypt */
	printf("%s,start encrypt...\n", __func__);
	ret = do_symc_crypto(&stCipherCtrl, pSrcVaddr, pDestVaddr, data_size, OP_ENCRYPT);
	if (ret != 0) {
		ES_ERR_CIPHER("%s, encrypt failed!\n", __func__);
		err++;
		goto FREE_DEST_BUF;
	}

	/* Decrypt */
	pDecryptSrcVaddr = pDestVaddr;
	ret = alloc_buf_and_fill_data(NULL, data_size, &pDecrytDestVaddr, &decryptDestMemFd, USER_MALLOC);
	if (ret != 0) {
		ES_ERR_CIPHER("%s, failed to alloc mem for dst data\n", __func__);
		err++;
		goto FREE_DEST_BUF;
	}

	printf("%s,start decrypt...\n", __func__);
	ret = do_symc_crypto(&stCipherCtrl, pDecryptSrcVaddr, pDecrytDestVaddr, data_size, OP_DECRYPT);
	if (ret != 0) {
		ES_ERR_CIPHER("%s, decrypt failed!\n", __func__);
		err++;
		goto FREE_DECRYPT_DEST_BUF;
	}

	/* compare the decrypted data with the cleartext src*/
	#ifndef ES_CIPHER_QEMU_DEBUG
	ret = memcmp(pSrcVaddr, pDecrytDestVaddr, data_size);
	if (ret) {
		ES_ERR_CIPHER("%s, decrypted data is NOT equal to the cleartxt src!!!\n", __func__);
		err++;
	}
	#endif

FREE_DECRYPT_DEST_BUF:
	free_data_buf(decryptDestMemFd, pDecrytDestVaddr, data_size);
FREE_DEST_BUF:
	free_data_buf(destMemFd, pDestVaddr, data_size);
FREE_SRC_BUF:
	free_data_buf(srcMemFd, pSrcVaddr, data_size);

	if (err) {
		ret = ES_FAILURE;
	}
	else {
		ret = ES_SUCCESS;

	}
	return ret;
}

/* multi package, encrypt and decrypt with external user key */
static ES_S32 multi_external_CBC_AES128(ES_VOID)
{
	ES_S32 i, ret = 0, err = 0;
	ES_CIPHER_CTRL_S stCipherCtrl;
	ES_CIPHER_CTRL_S *pstCipherCtrl = &stCipherCtrl;
	ES_U64 srcMemFd, destMemFd;
	ES_U64 decryptDestMemFd;
	ES_U8 *pSrcVaddr, *pDestVaddr;
	ES_U8 *pDecrytDestVaddr;
	int data_size = 0, package_len = 0;
	ES_HANDLE hCipherEnc, hCipherDec;

	printf("%s begin---\n", __func__);
	/* Init ES_CIPHER_CTRL_S */
	memset(&stCipherCtrl, 0, sizeof(ES_CIPHER_CTRL_S));

	stCipherCtrl.enKeySrc = ES_CIPHER_SYMC_KEY_SRC_USER;
	stCipherCtrl.enAlgMode = ES_CIPHER_ALG_MODE_AES_CBC;
	stCipherCtrl.enKeyLen = ES_CIPHER_KEY_AES_128BIT;
	stCipherCtrl.u32IvLen = 16;

	/* Init key and IV */
	memcpy(stCipherCtrl.u32Key, aes_key, ES_CIPHER_KEY_AES_128BIT);
	memcpy(stCipherCtrl.u32IV, aes_IV, stCipherCtrl.u32IvLen);

	data_size = sizeof(aes_src);
	/* alloc src and fill with test data, alloc dest buffer */
	ret = alloc_buf_and_fill_data(aes_src, data_size, &pSrcVaddr, &srcMemFd, HEAP_ALLOC);
	if (ret != 0) {
		ES_ERR_CIPHER("%s, failed to alloc mem for src data\n", __func__);
		return ES_FAILURE;
	}

	ret = alloc_buf_and_fill_data(NULL, data_size, &pDestVaddr, &destMemFd, USER_MALLOC);
	if (ret != 0) {
		ES_ERR_CIPHER("%s, failed to alloc mem for dst data\n", __func__);
		err++;
		goto FREE_SRC_BUF;
	}

	ret = alloc_buf_and_fill_data(NULL, data_size, &pDecrytDestVaddr, &decryptDestMemFd, USER_MALLOC);
	if (ret != 0) {
		ES_ERR_CIPHER("%s, failed to alloc mem for dst data\n", __func__);
		err++;
		goto FREE_DEST_BUF;
	}

	/* Encrypt */
	ret = ES_CIPHER_Init();
	if (ret != ES_SUCCESS) {
		ES_ERR_CIPHER("%s, ES_CIPHER_Init failed!\n", __func__);
		err++;
		goto FREE_DECRYPT_DEST_BUF;
	}

	ret = ES_CIPHER_CreateHandle(&hCipherEnc, pstCipherCtrl);
	if (ret != ES_SUCCESS) {
		printf("%s, create handle failed, err(%d)\n", __func__, ret);
		err++;
		goto CIPHER_DEINIT;
	}
	package_len = data_size/5;

	/* Encrypt */
	printf("%s,start encrypt...\n", __func__);
	for (i = 0; i < 5; i++) {
		ret = ES_CIPHER_Encrypt(hCipherEnc, pSrcVaddr + i*package_len, pDestVaddr + i*package_len, package_len);
		if (ret != 0) {
			ES_ERR_CIPHER("%s,  ES_CIPHER_Encrypt failed!\n", __func__);
			err++;
			goto CIPHER_EXIT_ENC;
		}
	}

	/* Decrypt */
	printf("%s,start decrypt...\n", __func__);
	memcpy(stCipherCtrl.u32Key, aes_key, ES_CIPHER_KEY_AES_128BIT);
	memcpy(stCipherCtrl.u32IV, aes_IV, stCipherCtrl.u32IvLen);
	ret = ES_CIPHER_CreateHandle(&hCipherDec, pstCipherCtrl);
	if (ret != ES_SUCCESS) {
		printf("%s, create handle failed, err(%d)\n", __func__, ret);
		err++;
		goto CIPHER_EXIT_ENC;
	}

	for (i = 0; i < 5; i++) {
		ret = ES_CIPHER_Decrypt(hCipherDec, pDestVaddr + i*package_len, pDecrytDestVaddr + i*package_len, package_len);
		if (ret != 0) {
			ES_ERR_CIPHER("%s,  ES_CIPHER_Decrypt failed!\n", __func__);
			err++;
			goto CIPHER_EXIT;
		}
	}

	/* compare the decrypted data with the cleartext src*/
	#ifndef ES_CIPHER_QEMU_DEBUG
	ret = memcmp(pSrcVaddr, pDecrytDestVaddr, data_size);
	if (ret) {
		ES_ERR_CIPHER("%s, decrypted data is NOT equal to the cleartxt src!!!\n", __func__);
		err++;
	}
	#endif

CIPHER_EXIT:
	ES_CIPHER_DestroyHandle(hCipherDec);
CIPHER_EXIT_ENC:
	ES_CIPHER_DestroyHandle(hCipherEnc);
CIPHER_DEINIT:
	ES_CIPHER_DeInit();
FREE_DECRYPT_DEST_BUF:
	free_data_buf(decryptDestMemFd, pDecrytDestVaddr, data_size);
FREE_DEST_BUF:
	free_data_buf(destMemFd, pDestVaddr, data_size);
FREE_SRC_BUF:
	free_data_buf(srcMemFd, pSrcVaddr, data_size);

	if (err) {
		ret = ES_FAILURE;
	}
	else {
		ret = ES_SUCCESS;

	}

	return ret;
}

int sample_aes_sm4(ES_VOID)
{
	ES_S32 s32Ret = ES_SUCCESS;
	ES_S32 err = 0;

	printf("Hello %s\n", __func__);

	s32Ret = external_CBC_AES128();
	if (s32Ret != ES_SUCCESS)
	{
		err++;
	}

	s32Ret = multi_external_CBC_AES128();
	if (s32Ret != ES_SUCCESS)
	{
		err++;
	}

	return err;
}