/******************************************************************************
Copyright (C) 2022  Beiing Eswin Co. Ltd
******************************************************************************
File Name     : sample_trng.c
Version       : Initial Draft
Author        : Eswin
Created       : 2023
Last Modified :
Description   : sample for true random generation
Function List :
History       :
******************************************************************************/
#include "sample_cipher_common.h"
typedef struct
{
	ES_CIHPER_ECC_CURVE_TYPE_E enEccCurve;
	char name[64];
	ES_CIPHER_ALG_MODE_E enShaType;
}ECDH_CURVE_TEST_DATA;
ECDH_CURVE_TEST_DATA ecdh_curve_test_data[] = {{ES_CIPHER_PKA_SW_CURVE_NIST_P256, "ES_CIPHER_PKA_SW_CURVE_NIST_P256"},
					       {ES_CIPHER_PKA_SW_CURVE_NIST_P384, "ES_CIPHER_PKA_SW_CURVE_NIST_P384"},
					       {ES_CIPHER_PKA_SW_CURVE_NIST_P521, "ES_CIPHER_PKA_SW_CURVE_NIST_P521"},
					       {ES_CIPHER_PKA_SW_CURVE_BRAINPOOL_P256R1, "ES_CIPHER_PKA_SW_CURVE_BRAINPOOL_P256R1"},
					       {ES_CIPHER_PKA_SW_CURVE_BRAINPOOL_P384R1, "ES_CIPHER_PKA_SW_CURVE_BRAINPOOL_P384R1"},
					       {ES_CIPHER_PKA_SW_CURVE_BRAINPOOL_P512R1, "ES_CIPHER_PKA_SW_CURVE_BRAINPOOL_P512R1"}};

static int ecdh_keybuf_alloc(ES_CIPHER_ECDH_KEY_S *pstEcdhKey)
{
	pstEcdhKey->pu8X = malloc(MAX_ECC_KEY_LEN);
	if (NULL == pstEcdhKey->pu8X) {
		return ES_FAILURE;
	}
	memset(pstEcdhKey->pu8X, 0, MAX_ECC_KEY_LEN);

	pstEcdhKey->pu8Y = malloc(MAX_ECC_KEY_LEN);
	if (NULL == pstEcdhKey->pu8Y) {
		free(pstEcdhKey->pu8X);
		return ES_FAILURE;
	}
	memset(pstEcdhKey->pu8Y, 0, MAX_ECC_KEY_LEN);

	pstEcdhKey->pu8Z = malloc(MAX_ECC_KEY_LEN);
	if (NULL == pstEcdhKey->pu8Z) {
		free(pstEcdhKey->pu8X);
		free(pstEcdhKey->pu8Y);
		return ES_FAILURE;
	}
	memset(pstEcdhKey->pu8Z, 0, MAX_ECC_KEY_LEN);

	return ES_SUCCESS;
}

static void ecdh_keybuf_free(ES_CIPHER_ECDH_KEY_S *pstEcdhKey)
{
	free(pstEcdhKey->pu8X);
	free(pstEcdhKey->pu8Y);
	free(pstEcdhKey->pu8Z);

}

static int ecdh_pubkey_buf_alloc(ES_CIPHER_ECDH_PUB_KEY_S *pstEcdhPubKey)
{
	pstEcdhPubKey->pu8X = malloc(MAX_ECC_KEY_LEN);
	if (NULL == pstEcdhPubKey->pu8X) {
		return ES_FAILURE;
	}
	memset(pstEcdhPubKey->pu8X, 0, MAX_ECC_KEY_LEN);

	pstEcdhPubKey->pu8Y = malloc(MAX_ECC_KEY_LEN);
	if (NULL == pstEcdhPubKey->pu8Y) {
		free(pstEcdhPubKey->pu8X);
		return ES_FAILURE;
	}
	memset(pstEcdhPubKey->pu8Y, 0, MAX_ECC_KEY_LEN);

	return ES_SUCCESS;
}

static void ecdh_pubkey_buf_free(ES_CIPHER_ECDH_PUB_KEY_S *pstEcdhPubKey)
{
	free(pstEcdhPubKey->pu8X);
	free(pstEcdhPubKey->pu8Y);

}

static int ecdh_prikey_buf_alloc(ES_CIPHER_ECDH_PRI_KEY_S *pstEcdhPriKey)
{
	pstEcdhPriKey->pu8Z = malloc(MAX_ECC_KEY_LEN);
	if (NULL == pstEcdhPriKey->pu8Z) {
		return ES_FAILURE;
	}
	memset(pstEcdhPriKey->pu8Z, 0, MAX_ECC_KEY_LEN);

	return ES_SUCCESS;
}

static void ecdh_prikey_buf_free(ES_CIPHER_ECDH_PRI_KEY_S *pstEcdhPriKey)
{
	free(pstEcdhPriKey->pu8Z);
}

int sample_ecdh(ES_VOID)
{
	ES_S32 i, total, ret = ES_SUCCESS;
	ES_CIPHER_ECDH_KEY_S stEcdhKey_A;
	ES_CIPHER_ECDH_KEY_S stEcdhKey_B;
	ES_CIPHER_ECDH_PRI_KEY_S stLocalEcdhPriKey_A;
	ES_CIPHER_ECDH_PRI_KEY_S stLocalEcdhPriKey_B;
	ES_CIPHER_ECDH_PUB_KEY_S stOutSharedEcdhPubKey_A;
	ES_CIPHER_ECDH_PUB_KEY_S stOutSharedEcdhPubKey_B;
	ES_U8 u8EccSymcKey_A[ECC_SYMC_KEY_LEN];
	ES_U8 u8EccSymcKey_B[ECC_SYMC_KEY_LEN];
	ES_S32 u32Rand;


	printf("Hello %s\n", __func__);

	ret = ES_CIPHER_Init();
	if (ret != ES_SUCCESS) {
		ES_ERR_CIPHER("%s, ES_CIPHER_Init failed!\n", __func__);
		return ret;
	}
	/* alloc ecdh key buf*/
	ret = ecdh_keybuf_alloc(&stEcdhKey_A);
	if (ES_SUCCESS != ret) {
		ES_ERR_CIPHER("%s, failed to alloc ECDH key buf for peer A!\n", __func__);
		goto CIPHER_EXIT;
	}

	ret = ecdh_keybuf_alloc(&stEcdhKey_B);
	if (ES_SUCCESS != ret) {
		ES_ERR_CIPHER("%s, failed to alloc ECDH key buf for peer B!\n", __func__);
		goto FREE_A_KEY_BUF;
	}

	/* alloc buf for output shared pub key */
	ret = ecdh_pubkey_buf_alloc(&stOutSharedEcdhPubKey_A);
	if (ES_SUCCESS != ret) {
		ES_ERR_CIPHER("%s, failed to alloc ECDH sharedPubKey buf for peer A!\n", __func__);
		goto FREE_B_KEY_BUF;
	}

	ret = ecdh_pubkey_buf_alloc(&stOutSharedEcdhPubKey_B);
	if (ES_SUCCESS != ret) {
		ES_ERR_CIPHER("%s, failed to alloc ECDH sharedPubKey buf for peer A!\n", __func__);
		goto FREE_A_SHARED_PUBKEY_BUF;
	}

	/* alloc buf for storing the local pri key */
	ret = ecdh_prikey_buf_alloc(&stLocalEcdhPriKey_A);
	if (ES_SUCCESS != ret) {
		ES_ERR_CIPHER("%s, failed to alloc buf for storing pri key of peer A!\n", __func__);
		goto FREE_B_SHARED_PUBKEY_BUF;
	}

	ret = ecdh_prikey_buf_alloc(&stLocalEcdhPriKey_B);
	if (ES_SUCCESS != ret) {
		ES_ERR_CIPHER("%s, failed to alloc buf for storing pri key of peer B!\n", __func__);
		goto FREE_A_PRIKEY_BUF;
	}

	total = sizeof(ecdh_curve_test_data)/sizeof(ECDH_CURVE_TEST_DATA);
	for (i = 0; i < total; i++) {
		printf("---%s test(%d/%d)---\n", ecdh_curve_test_data[i].name, i, total);
		/* ---ECC Pub key and Priv key gen */
		printf("pub key and priv key generating...\n");
		ret = ES_CIPHER_EcdhKeyGen(ecdh_curve_test_data[i].enEccCurve, &stEcdhKey_A);
		if (ES_SUCCESS != ret) {
			ES_ERR_CIPHER("%s, ES_CIPHER_EcdhKeyGen failed! errCode:0x%x\n", __func__, ret);
			break;
		}
		memcpy(stLocalEcdhPriKey_A.pu8Z, stEcdhKey_A.pu8Z, stEcdhKey_A.keylen);
		stLocalEcdhPriKey_A.keylen = stEcdhKey_A.keylen;

		ret = ES_CIPHER_EcdhKeyGen(ecdh_curve_test_data[i].enEccCurve, &stEcdhKey_B);
		if (ES_SUCCESS != ret) {
			ES_ERR_CIPHER("%s, ES_CIPHER_EcdhKeyGen failed! errCode:0x%x\n", __func__, ret);
			break;
		}
		memcpy(stLocalEcdhPriKey_B.pu8Z, stEcdhKey_B.pu8Z, stEcdhKey_B.keylen);
		stLocalEcdhPriKey_B.keylen = stEcdhKey_B.keylen;

		/* ---Get random data for negotiation */
		printf("u32Rand generating...\n");
		ret = ES_CIPHER_GetRandomNumber((ES_U8 *)&u32Rand, sizeof(u32Rand));
		if (ES_SUCCESS != ret) {
			ES_ERR_CIPHER("%s, ES_CIPHER_GetRandomNumber failed! errCode:0x%x\n", __func__, ret);
			break;
		}
		/* ---ECC key agreement and derivation*/
		printf("ECC key agreement and derivation...\n");
		/* A */
		ret = ES_CIPHER_EcdhKeyAgree(ecdh_curve_test_data[i].enEccCurve, (ES_CIPHER_ECDH_PUB_KEY_S *)&stEcdhKey_B, &stLocalEcdhPriKey_A, &stOutSharedEcdhPubKey_A);
		if (ES_SUCCESS != ret) {
			ES_ERR_CIPHER("%s, ES_CIPHER_EcdhKeyAgree for A failed! errCode:0x%x\n", __func__, ret);
			break;
		}
		ret = ES_CIPHER_EcdhKeyDerivation(ecdh_curve_test_data[i].enEccCurve, &stOutSharedEcdhPubKey_A, u32Rand, u8EccSymcKey_A);
		if (ES_SUCCESS != ret) {
			ES_ERR_CIPHER("%s, ES_CIPHER_EcdhKeyDerivation for A failed! errCode:0x%x\n", __func__, ret);
			break;
		}

		/* B */
		ret = ES_CIPHER_EcdhKeyAgree(ecdh_curve_test_data[i].enEccCurve, (ES_CIPHER_ECDH_PUB_KEY_S *)&stEcdhKey_A, &stLocalEcdhPriKey_B, &stOutSharedEcdhPubKey_B);
		if (ES_SUCCESS != ret) {
			ES_ERR_CIPHER("%s, ES_CIPHER_EcdhKeyAgree for B failed! errCode:0x%x\n", __func__, ret);
			break;
		}
		ret = ES_CIPHER_EcdhKeyDerivation(ecdh_curve_test_data[i].enEccCurve, &stOutSharedEcdhPubKey_B, u32Rand, u8EccSymcKey_B);
		if (ES_SUCCESS != ret) {
			ES_ERR_CIPHER("%s, ES_CIPHER_EcdhKeyDerivation for B failed! errCode:0x%x\n", __func__, ret);
			break;
		}
		/* ---check if the key is expected */
		/* print A shared pub key and EccSymcKey */
		printf("stOutSharedEcdhPubKey_A.pu8X(len=0x%x) is:\n", stOutSharedEcdhPubKey_A.keylen);
		print_data(stOutSharedEcdhPubKey_A.pu8X, stOutSharedEcdhPubKey_A.keylen);

		printf("stOutSharedEcdhPubKey_A.pu8Y(len=0x%x) is:\n", stOutSharedEcdhPubKey_A.keylen);
		print_data(stOutSharedEcdhPubKey_A.pu8Y, stOutSharedEcdhPubKey_A.keylen);

		printf("u8EccSymcKey_A(len=0x%x) is:\n", ECC_SYMC_KEY_LEN);
		print_data(u8EccSymcKey_A, ECC_SYMC_KEY_LEN);

		/* print B shared pub key and EccSymcKey */
		printf("stOutSharedEcdhPubKey_B.pu8X(len=0x%x) is:\n", stOutSharedEcdhPubKey_B.keylen);
		print_data(stOutSharedEcdhPubKey_B.pu8X, stOutSharedEcdhPubKey_B.keylen);

		printf("stOutSharedEcdhPubKey_B.pu8Y(len=0x%x) is:\n", stOutSharedEcdhPubKey_B.keylen);
		print_data(stOutSharedEcdhPubKey_B.pu8Y, stOutSharedEcdhPubKey_B.keylen);

		printf("u8EccSymcKey_B(len=0x%x) is:\n", ECC_SYMC_KEY_LEN);
		print_data(u8EccSymcKey_B, ECC_SYMC_KEY_LEN);

		if (stOutSharedEcdhPubKey_A.keylen != stOutSharedEcdhPubKey_B.keylen) {
			ES_ERR_CIPHER("%s, stOutSharedEcdhPubKey_A.keylen is not equal to stOutSharedEcdhPubKey_B.keylen\n", __func__);
			ret = ES_FAILURE;
			break;
		}
		if (memcmp(stOutSharedEcdhPubKey_A.pu8X, stOutSharedEcdhPubKey_B.pu8X, stOutSharedEcdhPubKey_A.keylen)) {
			ES_ERR_CIPHER("%s, stOutSharedEcdhPubKey_A.pu8X is not equal to stOutSharedEcdhPubKey_B.pu8X\n", __func__);
			ret = ES_FAILURE;
			break;
		}
		if (memcmp(stOutSharedEcdhPubKey_A.pu8Y, stOutSharedEcdhPubKey_B.pu8Y, stOutSharedEcdhPubKey_A.keylen)) {
			ES_ERR_CIPHER("%s, stOutSharedEcdhPubKey_A.pu8Y is not equal to stOutSharedEcdhPubKey_B.pu8Y\n", __func__);
			ret = ES_FAILURE;
			break;
		}
		if (memcmp(u8EccSymcKey_A, u8EccSymcKey_B, ECC_SYMC_KEY_LEN)) {
			ES_ERR_CIPHER("%s, u8EccSymcKey_A is not equal to u8EccSymcKey_B\n", __func__);
			ret = ES_FAILURE;
			break;
		}
	}
	//FREE_B_PRIKEY_BUF:
	ecdh_prikey_buf_free(&stLocalEcdhPriKey_B);
FREE_A_PRIKEY_BUF:
	ecdh_prikey_buf_free(&stLocalEcdhPriKey_A);

FREE_B_SHARED_PUBKEY_BUF:
	ecdh_pubkey_buf_free(&stOutSharedEcdhPubKey_B);
FREE_A_SHARED_PUBKEY_BUF:
	ecdh_pubkey_buf_free(&stOutSharedEcdhPubKey_A);

FREE_B_KEY_BUF:
	ecdh_keybuf_free(&stEcdhKey_B);
FREE_A_KEY_BUF:
	ecdh_keybuf_free(&stEcdhKey_A);

CIPHER_EXIT:
	ES_CIPHER_DeInit();

	return ret;


}