/******************************************************************************
Copyright (C) 2022  Beiing Eswin Co. Ltd
******************************************************************************
File Name     : sample_hash.c
Version       : Initial Draft
Author        : Eswin
Created       : 2023
Last Modified :
Description   : sample for hash
Function List :
History       :
******************************************************************************/
#include "sample_cipher_common.h"

ES_U8 hash_txt_8byte[8]="12345678";
/* sha256 8byte digest golden result */
ES_U8 sha256_result[32] = { 0xef, 0x79, 0x7c, 0x81, 0x18, 0xf0, 0x2d, 0xfb,
			0x64, 0x96, 0x07, 0xdd, 0x5d, 0x3f, 0x8c, 0x76,
			0x23, 0x04, 0x8c, 0x9c, 0x06, 0x3d, 0x53, 0x2c,
			0xc9, 0x5c, 0x5e, 0xd7, 0xa8, 0x98, 0xa6, 0x4f };

/* sha1 8byte digest golden result */
ES_U8 sha1_result[20] = { 0x7c, 0x22, 0x2f, 0xb2, 0x92, 0x7d, 0x82,
			0x8a, 0xf2, 0x2f, 0x59, 0x21, 0x34, 0xe8,
			0x93, 0x24, 0x80, 0x63, 0x7c, 0x0d };

/* md5 8byte digest golden result */
ES_U8 md5_result[16] = { 0x25, 0xd5, 0x5a, 0xd2, 0x83, 0xaa, 0x40, 0x0a,
			0xf4, 0x64, 0xc7, 0x6d, 0x71, 0x3c, 0x07, 0xad };

/* HMAC key */
ES_U8 hmac_key[16] = "0123456789abcdef";

/* HMAC_SHA256 golden result generated with <echo -n "12345678" | openssl dgst -sha256 -hmac "0123456789abcdef"> */
ES_U8 hmac_sha256_result[32] = {0xc8, 0xbc, 0x65, 0x6c, 0x80, 0x23, 0x5b, 0x84,
				0xf3, 0x54, 0x6c, 0x1a, 0xfb, 0x3d, 0xae, 0x76,
				0x11, 0xaf, 0xdb, 0x57, 0x5a, 0xd8, 0x1e, 0xf3,
				0xeb, 0x0f, 0xa0, 0x25, 0x7c, 0x1f, 0xbf, 0xa2};

/* HMAC_SM3 golden result generated with <echo -n "12345678" | openssl dgst -sm3 -hmac "0123456789abcdef"> */
ES_U8 hmac_sm3_result[32] = {0x60, 0x5b, 0x58, 0x7d, 0xd9, 0xd5, 0x43, 0xb6,
			     0x86, 0x58, 0x90, 0xfc, 0xdd, 0x89, 0xaf, 0x52,
			     0xce, 0x0a, 0xc3, 0x0e, 0xeb, 0x12, 0xfa, 0x0b,
			     0xe2, 0xd1, 0x6b, 0xfb, 0x15, 0x90, 0x9a, 0x84};

static ES_S32 do_sha_or_hmac(ES_CIPHER_HASH_CTRL_S *pstHashCtrl,ES_U8 *pu8InputData, ES_U32 u32InputDataLen, ES_U8 *pu8OutputHashExpected, ES_U32 u32OutputHashLen)
{
	ES_S32 ret = ES_FAILURE, err = 0;
	ES_HANDLE hHashHandle = -1;
	ES_U64 srcMemFd;
	ES_U8 *pSrcVaddr = NULL;
	ES_U8 *pu8OutputHash = NULL;
	ES_U32 u32HlenResult;

	pu8OutputHash = malloc(u32OutputHashLen);
	if (NULL == pu8OutputHash) {
		printf("failed to malloc outputHash buffer!!!\n");
		err++;
		return ES_FAILURE;
	}

	ret = alloc_buf_and_fill_data(pu8InputData, u32InputDataLen, &pSrcVaddr, &srcMemFd, HEAP_ALLOC);
	if (ret != 0) {
		printf("%s, failed to alloc mem for dst data\n", __func__);
		free(pu8OutputHash);
		err++;
		return ret;
	}

	ret = ES_CIPHER_Init();
	if (ret != ES_SUCCESS) {
		printf("%s, failed to ES_CIPHER_Init\n", __func__);
		err++;
		goto FREE_SRC_BUF;
	}

	ret = ES_CIPHER_HashInit(pstHashCtrl, &hHashHandle);
	if (ret != ES_SUCCESS) {
		err++;
		goto CIPHER_EXIT;
	}

	ret = ES_CIPHER_HashFinish(hHashHandle, pSrcVaddr, u32InputDataLen, pu8OutputHash, &u32HlenResult);
	if (ret != ES_SUCCESS) {
		err++;
		goto CIPHER_EXIT;
	}
	#ifndef ES_CIPHER_QEMU_DEBUG
	printf("digest_size     = [0x%08x]\r\n", u32HlenResult);

	for (int i = 0; i < u32HlenResult; i++) {
		printf("[%d]=0x%02x(0x%02x)\r\n", i, pu8OutputHash[i], pu8OutputHashExpected[i]);
		if (pu8OutputHash[i] != pu8OutputHashExpected[i]) {
			printf("\r\ndigest[%d] data is not right!!!\r\n", i);
			err++;
		}
	}
	#endif

CIPHER_EXIT:
	ES_CIPHER_DeInit();
FREE_SRC_BUF:
	free_data_buf(srcMemFd, pSrcVaddr, u32InputDataLen);

	free(pu8OutputHash);

	if (err) {
		ret = ES_FAILURE;
	}
	else {
		ret = ES_SUCCESS;

	}

	return ret;
}

ES_S32 SHA1(void)
{
	ES_S32 ret = ES_FAILURE;
	ES_CIPHER_HASH_CTRL_S stHashCtrl;

	printf("%s begin---\n", __func__);
	stHashCtrl.enShaType = ES_CIPHER_ALG_MODE_HASH_SHA1;
	ret = do_sha_or_hmac(&stHashCtrl, hash_txt_8byte, sizeof(hash_txt_8byte), sha1_result, sizeof(sha1_result));
	if (ret != ES_SUCCESS) {
		ES_ERR_CIPHER("%s failed!\n", __func__);
	}

	return ret;
}

ES_S32 SHA256(void)
{
	ES_S32 ret = ES_FAILURE;
	ES_CIPHER_HASH_CTRL_S stHashCtrl;

	printf("%s begin---\n", __func__);
	stHashCtrl.enShaType = ES_CIPHER_ALG_MODE_HASH_SHA256;
	ret = do_sha_or_hmac(&stHashCtrl, hash_txt_8byte, sizeof(hash_txt_8byte), sha256_result, sizeof(sha256_result));
	if (ret != ES_SUCCESS) {
		ES_ERR_CIPHER("%s failed!\n", __func__);
	}

	return ret;
}

ES_S32 MD5(void)
{
	ES_S32 ret = ES_FAILURE;
	ES_CIPHER_HASH_CTRL_S stHashCtrl;

	printf("%s begin---\n", __func__);
	stHashCtrl.enShaType = ES_CIPHER_ALG_MODE_HASH_MD5;
	ret = do_sha_or_hmac(&stHashCtrl, hash_txt_8byte, sizeof(hash_txt_8byte), md5_result, sizeof(md5_result));
	if (ret != ES_SUCCESS) {
		ES_ERR_CIPHER("%s failed!\n", __func__);
	}

	return ret;
}

ES_S32 HMAC_SHA256(void)
{
	ES_S32 ret = ES_FAILURE;
	ES_CIPHER_HASH_CTRL_S stHashCtrl;

	printf("%s begin---\n", __func__);
	stHashCtrl.enShaType = ES_CIPHER_ALG_MODE_HMAC_SHA256;
	stHashCtrl.pu8HMACKey = hmac_key;
	stHashCtrl.u32HMACKeyLen = sizeof(hmac_key);
	ret = do_sha_or_hmac(&stHashCtrl, hash_txt_8byte, sizeof(hash_txt_8byte), hmac_sha256_result, sizeof(hmac_sha256_result));
	if (ret != ES_SUCCESS) {
		ES_ERR_CIPHER("%s failed!\n", __func__);
	}

	return ret;
}

ES_S32 HMAC_SM3(void)
{
	ES_S32 ret = ES_FAILURE;
	ES_CIPHER_HASH_CTRL_S stHashCtrl;

	printf("%s begin---\n", __func__);
	stHashCtrl.enShaType = ES_CIPHER_ALG_MODE_HMAC_SM3;
	stHashCtrl.pu8HMACKey = hmac_key;
	stHashCtrl.u32HMACKeyLen = sizeof(hmac_key);
	ret = do_sha_or_hmac(&stHashCtrl, hash_txt_8byte, sizeof(hash_txt_8byte), hmac_sm3_result, sizeof(hmac_sm3_result));
	if (ret != ES_SUCCESS) {
		ES_ERR_CIPHER("%s failed!\n", __func__);
	}

	return ret;
}

int sample_hash(ES_VOID)
{
	ES_S32 s32Ret = ES_SUCCESS;
	ES_S32 err = 0;

	printf("Hello %s\n", __func__);

	s32Ret = SHA1();
	if (s32Ret != ES_SUCCESS)
	{
		err++;
	}

	s32Ret = SHA256();
	if (s32Ret != ES_SUCCESS)
	{
		err++;
	}

	s32Ret = MD5();
	if (s32Ret != ES_SUCCESS)
	{
		err++;
	}

	s32Ret = HMAC_SHA256();
	if (s32Ret != ES_SUCCESS)
	{
		err++;
	}

	s32Ret = HMAC_SM3();
	if (s32Ret != ES_SUCCESS)
	{
		err++;
	}

	return err;
}