/******************************************************************************
Copyright (C) 2022  Beiing Eswin Co. Ltd
******************************************************************************
File Name     : sample_image_dec.c
Version       : Initial Draft
Author        : Eswin
Created       : 2023
Last Modified :
Description   : sample for image decryption
Function List :
History       :
******************************************************************************/
#include "sample_cipher_common.h"
/* This nsign image was generated from sample_cipher_data_image.data by nsign tool
   with the parameters listed below: 
   sign_algorithm = rsa
   keyid = 07
   versioin = <default>
   link_addr = <default>
   load_addr = <default>
   entry_addr = <default>
   payload_flags = plaintext
   digest_mthd = SHA
   encrypted_mthd = plaintext
   payload_type = BOOTLOADER
   boot_flags = U84
   devid = 0000000000000000
   vid = 00
   last_flag = yes
   lang = <default>
   mid = 0000000000000000
   reserved0 = <default>
   ...
   ...
   dl_irq_ofs =< default>
*/
#define IMAGE_NSIGN_FILE_PATH "/opt/eswin/data/cipher/sample_cipher_data_image_nsign.data"

/* Notes:
   This sample is to show the users about how to use the Cipher LIB Api.
   This sample will always fail actually since the symetric algorithm key(AES or SM4) in OTP is left for programming
   by users. And the encrypted_mthd parameter MUST be AES or SM4, but in this case it is plaintext. So the
   ES_CIPHER_ImageDec() API will always return  "service_status:-79950", it means ERR_INVALID_SIGN.
*/
int sample_image_dec(ES_VOID)
{
	ES_S32 ret = ES_FAILURE;
	ES_U8 *pu8InFileData = NULL;
	ES_S32 fileSize = 0;
	ES_CIPHER_NSIGN_VERIFY_S stNsignVerify;
	ES_U32 outpayloadLen = 0;
	ES_S32 err = 0;
	ES_U8 *pSrcVaddr, *pDestVaddr, *pSignVaddr;
	ES_U32 signLen, payloadLen;

	printf("%s begin---\n", __func__);
	memset(&stNsignVerify, 0, sizeof(stNsignVerify));

	fileSize = file_size(IMAGE_NSIGN_FILE_PATH);
	if (fileSize < 0) {
		printf("failed to get file size!!!\n");
		return ES_FAILURE;
	}
	pu8InFileData = malloc(fileSize);
	if (!pu8InFileData) {
		ES_ERR_CIPHER("%s, failed to malloc buffer for reading file!!!\n", __func__);
		return ES_FAILURE;
	}
	/* read file to buffer */
	ret = read_file_to_buf(IMAGE_NSIGN_FILE_PATH, pu8InFileData);
	if (ret) {
		ES_ERR_CIPHER("%s, failed to read file!!!\n", __func__);
		ret = ES_FAILURE;
		goto OUT_Free_pu8InFileData;
	}

	ret = ES_CIPHER_NsignFileParse(pu8InFileData, fileSize,
					&stNsignVerify,
					&pSignVaddr, &signLen,
					&pSrcVaddr, &payloadLen);
	if (ret) {
		ES_ERR_CIPHER("%s, failed to parse nsign file!!!\n", __func__);
		ret = ES_FAILURE;
		goto OUT_Free_pu8InFileData;
	}
	printf("%s signLen:0x%x, payloadLen:0x%x\n", __func__, signLen, payloadLen);

	/* Since the image is encrypted with symetric algorithm, the size of the decrypted image 
	   is equal to the size of the encrypted image */
	pDestVaddr = malloc(payloadLen);
	if (!pDestVaddr) {
		ES_ERR_CIPHER("%s, failed to malloc buffer for output decrypted data!!!\n", __func__);
		ret = ES_FAILURE;
		goto OUT_Free_pu8InFileData;
	}
	memset(pDestVaddr, 0, payloadLen);

	ret = ES_CIPHER_Init();
	if ( ES_SUCCESS != ret )
	{
		ES_ERR_CIPHER("%s, ES_CIPHER_Init failed.\n", __func__);
		err++;
		goto OUT;
	}

	stNsignVerify.enScheme = ES_CIPHER_RSA_SCHEME_RSA2048;
	stNsignVerify.enKeySrc = ES_CIPHER_RSA_PUB_KEY_SRC_ROM_7;
	ret = ES_CIPHER_ImageDec(&stNsignVerify,
					(ES_U8 *)pSrcVaddr, payloadLen,
					(ES_U8 *)pSignVaddr, signLen,
					pDestVaddr, &outpayloadLen);

	if (ES_SUCCESS != ret) {
		ES_ERR_CIPHER("%s, failed\n", __func__);
		err++;
	}

	ret = ES_CIPHER_DeInit();
	if ( ES_SUCCESS != ret )
	{
		ES_ERR_CIPHER("%s, ES_CIPHER_DeInit failed.\n", __func__);
		err++;
	}
OUT:
	free(pDestVaddr);
	if (err) {
		ret = ES_FAILURE;
	}
	else {
		ret = ES_SUCCESS;

	}
OUT_Free_pu8InFileData:
	free(pu8InFileData);

	return ret;
}